# This file is part of loadRunner2jmeter.
#
# loadRunner2jmeter is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# loadRunner2jmeter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with loadRunner2jmeter.  If not, see <http://www.gnu.org/licenses/>.
#
#(C)Copyright 2011 WatchMouse

package LR2JMParser;

use strict;
use warnings;

sub parse_file {
	my ($class, @filenames) = @_;

	@filenames									or die "input filename required";

	my $file = '';
	foreach my $filename (@filenames) {
		open FILE, $filename					or die "Couldn't open file '$filename' for reading: $!";
		$file .= do { local $/; <FILE> };
		close FILE;
	}

	# De-comment
	$file =~ s/^\s*\/\/.*$//mg;
	$file =~ s/\/\*.*?\*\///sg;

	# Remove 'else' clauses
	my $num_elses = $file =~ s/\belse\s*\{.*?\}\s*\n//sg;
	print "'else' clauses found in C source(s) and eliminated: $num_elses\n\n" if $num_elses;

	my $simple_literal_string_regexp = qr/"(?:\\"|[^"])*"/;
	my $literal_string_regexp = qr/$simple_literal_string_regexp(?:\s*$simple_literal_string_regexp)*/;
	my $decimal_number_regexp = qr/\d+/;
	my $predeclared_constant_regexp = qr/\b[A-Z][A-Z_]*\b/;
	my $comment1_regexp = qr/\/\/.*$/m;
	my $comment2_regexp = qr/\/\*.*?\*\//s;
	my $function_names_regexp = qr/[a-zA-Z_][a-zA-Z0-9_]*/;
	my $any_value_regexp = qr/(?:$literal_string_regexp|$decimal_number_regexp|$predeclared_constant_regexp)/;

	my @matches = $file =~ /$function_names_regexp\s*\(\s*(?:$any_value_regexp\s*)?(?:\,\s*$any_value_regexp\s*)*\)\s*\S/g;
	@matches = grep { substr($_, -1, 1) ne '{' } @matches;
	
	my @output;

	foreach my $match (@matches) {
		$match =~ s/($function_names_regexp)//;
		my $function_name = $1;

		$match =~ s/^\s*\(\s*(.*?)\s*\)\s*\S$/$1/s;
		my @params = $match =~ /$any_value_regexp/g;
		foreach my $param (@params) {
			if ($param =~ /^$literal_string_regexp$/ and $param !~ /^$simple_literal_string_regexp$/) {
				my @simples = $param =~ /$simple_literal_string_regexp/g;
				foreach my $simple (@simples) {
					$simple =~ s/^"(.*)"$/$1/;
				}
				$param = '"' . join('', @simples) . '"';
				$param =~ /^$simple_literal_string_regexp$/		or die "strange string";
			}
		}
		push @output, $function_name;
		push @output, map { "\t$_" } @params;
		push @output, '';
	}

	return @output;
}

1;